/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-

   Copyright (C) 2004-2006 Bastien Nocera <hadess@hadess.net>

   The Gnome Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Gnome Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the Gnome Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Author: Bastien Nocera <hadess@hadess.net>
   Adapted by Duffman 2007
 */


#include "config.h"
#ifdef WITH_DBUS

#define DBUS_API_SUBJECT_TO_CHANGE 1
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

#define GS_SERVICE   "org.gnome.ScreenSaver"
#define GS_PATH      "/org/gnome/ScreenSaver"
#define GS_INTERFACE "org.gnome.ScreenSaver"

#include "gnomescrsaver.h"

static GObjectClass *parent_class = NULL;
static void xdtvgnome_scrsaver_class_init (XdTVGnomeScrsaverClass *class);
static void xdtvgnome_scrsaver_init       (XdTVGnomeScrsaver      *parser);
static void xdtvgnome_scrsaver_finalize   (GObject *object);


struct XdTVGnomeScrsaverPrivate {
  /* Whether the screensaver is disabled */
  gboolean disabled;

  DBusGConnection *connection;
  DBusGProxy *gs_proxy;
  guint32 cookie;
};

G_DEFINE_TYPE(XdTVGnomeScrsaver, xdtvgnome_scrsaver, G_TYPE_OBJECT)

static gboolean
screensaver_is_running_dbus (XdTVGnomeScrsaver *scr)
{
  if (! scr->priv->connection)
    return FALSE;

  if (! scr->priv->gs_proxy)
    return FALSE;

  return TRUE;
}

static void
screensaver_inhibit_dbus (XdTVGnomeScrsaver *scr,
                          gboolean	 inhibit)
{
  GError *error;
  gboolean res;

  g_return_if_fail (scr != NULL);
  g_return_if_fail (scr->priv->connection != NULL);
  g_return_if_fail (scr->priv->gs_proxy != NULL);

  error = NULL;
  if (inhibit) {
    char   *application;
    char   *reason;
    guint32 cookie;

    application = g_strdup ("XdTV");
    reason = g_strdup ("Watching TV");

    res = dbus_g_proxy_call (scr->priv->gs_proxy,
                             "Inhibit",
                             &error,
                             G_TYPE_STRING, application,
                             G_TYPE_STRING, reason,
                             G_TYPE_INVALID,
                             G_TYPE_UINT, &cookie,
                             G_TYPE_INVALID);

    if (res) {
      /* save the cookie */
      scr->priv->cookie = cookie;
    } else {
      /* try the old API */
      res = dbus_g_proxy_call (scr->priv->gs_proxy,
                               "InhibitActivation",
                               &error,
                               G_TYPE_STRING, reason,
                               G_TYPE_INVALID,
                               G_TYPE_INVALID);
    }

    g_free (reason);
    g_free (application);

  } else {
    res = dbus_g_proxy_call (scr->priv->gs_proxy,
                             "UnInhibit",
                             &error,
                             G_TYPE_UINT, scr->priv->cookie,
                             G_TYPE_INVALID,
                             G_TYPE_INVALID);
    if (res) {
      /* clear the cookie */
      scr->priv->cookie = 0;
    } else {
      /* try the old API */
      res = dbus_g_proxy_call (scr->priv->gs_proxy,
                               "AllowActivation",
                               &error,
                               G_TYPE_INVALID,
                               G_TYPE_INVALID);
    }
  }

  if (! res) {
    if (error) {
      g_warning ("Problem inhibiting the screensaver: %s", error->message);
      g_error_free (error);
    }
  }
}

static void
gs_proxy_destroy_cb (GObject *proxy,
                     XdTVGnomeScrsaver *scr)
{
  g_warning ("Detected that GNOME screensaver has left the bus");

  /* just invalidate for now */
  scr->priv->gs_proxy = NULL;
}

static void
xdtvgnome_scrsaver_class_init (XdTVGnomeScrsaverClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = xdtvgnome_scrsaver_finalize;
}

XdTVGnomeScrsaver *
xdtvgnome_scrsaver_new (void)
{
  return XDTVGNOME_SCRSAVER (g_object_new (XDTVGNOME_TYPE_SCRSAVER, NULL));
}

static void
xdtvgnome_scrsaver_init (XdTVGnomeScrsaver *scr)
{
  GError *error = NULL;

  scr->priv = g_new0 (XdTVGnomeScrsaverPrivate, 1);

  scr->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);

  if (! scr->priv->connection) {
    if (error) {
      g_warning ("Failed to connect to the session bus: %s", error->message);
      g_error_free (error);
    }
    return;
  }

  scr->priv->gs_proxy = dbus_g_proxy_new_for_name_owner (scr->priv->connection,
                                                         GS_SERVICE,
                                                         GS_PATH,
                                                         GS_INTERFACE,
                                                         NULL);
  if (scr->priv->gs_proxy != NULL) {
    g_signal_connect_object (scr->priv->gs_proxy,
                             "destroy",
                             G_CALLBACK (gs_proxy_destroy_cb),
                             scr,
                             0);

  }
}

void
xdtvgnome_scrsaver_disable (XdTVGnomeScrsaver *scr)
{
  if (scr->priv->disabled != FALSE)
    return;

  scr->priv->disabled = TRUE;

  if (screensaver_is_running_dbus (scr) != FALSE)
    screensaver_inhibit_dbus (scr, TRUE);
}

void
xdtvgnome_scrsaver_enable (XdTVGnomeScrsaver *scr)
{
  if (scr->priv->disabled == FALSE)
    return;

  scr->priv->disabled = FALSE;

  if (screensaver_is_running_dbus (scr) != FALSE)
    screensaver_inhibit_dbus (scr, FALSE);
}

static void
xdtvgnome_scrsaver_finalize (GObject *object)
{
  XdTVGnomeScrsaver *scr = XDTVGNOME_SCRSAVER (object);

  if (scr->priv->gs_proxy) {
    g_object_unref (scr->priv->gs_proxy);
  }

  g_free (scr->priv);

  if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
  }
}

#endif /* WITH_DBUS */
